
STRINGTABLE 
{
    1, "Out of Memory"
    2, "New SSL Sniffer Server Registered"
    3, "Checking IP Header"
    4, "SSL Sniffer Server Not Registered"
    5, "Checking TCP Header"

    6, "SSL Sniffer Server Port Not Registered"
    7, "RSA Private Decrypt Error"
    8, "RSA Private Decode Error"
    9, "Set Cipher Spec Error"
    10, "Server Hello Input Malformed"

    11, "Couldn't Resume Session Error"
    12, "Server Did Resumption"
    13, "Client Hello Input Malformed"
    14, "Client Trying to Resume"
    15, "Handshake Input Malformed"

    16, "Got Hello Verify msg"
    17, "Got Server Hello msg"
    18, "Got Cert Request msg"
    19, "Got Server Key Exchange msg"
    20, "Got Cert msg"

    21, "Got Server Hello Done msg"
    22, "Got Finished msg"
    23, "Got Client Hello msg"
    24, "Got Client Key Exchange msg"
    25, "Got Cert Verify msg"

    26, "Got Unknown Handshake msg"
    27, "New SSL Sniffer Session created"
    28, "Couldn't create new SSL"
    29, "Got a Packet to decode"
    30, "No data present"

    31, "Session Not Found"
    32, "Got an Old Client Hello msg"
    33, "Old Client Hello Input Malformed"
    34, "Old Client Hello OK"
    35, "Bad Old Client Hello"

    36, "Bad Record Header"
    37, "Record Header Input Malformed"
    38, "Got a HandShake msg"
    39, "Bad HandShake msg"
    40, "Got a Change Cipher Spec msg"

    41, "Got Application Data msg"
    42, "Bad Application Data"
    43, "Got an Alert msg"
    44, "Another msg to Process"
    45, "Removing Session From Table"

    46, "Bad Key File"
    47, "Wrong IP Version"
    48, "Wrong Protocol type"
    49, "Packet Short for header processing"
    50, "Got Unknown Record Type"
    
    51, "Can't Open Trace File"
    52, "Session in Fatal Error State"
    53, "Partial SSL record received"
    54, "Buffer Error, malformed input"
    55, "Added to Partial Input"

    56, "Received a Duplicate Packet"
    57, "Received an Out of Order Packet"
    58, "Received an Overlap Duplicate Packet"
    59, "Received an Overlap Reassembly Begin Duplicate Packet"
    60, "Received an Overlap Reassembly End Duplicate Packet"
    
    61, "Missed the Client Hello Entirely"
    62, "Got Hello Request msg"
    63, "Got Session Ticket msg"
    64, "Bad Input"
    65, "Bad Decrypt Type"

    66, "Bad Finished Message Processing"
    67, "Bad Compression Type"
    68, "Bad DeriveKeys Error"
    69, "Saw ACK for Missing Packet Error"
    70, "Bad Decrypt Operation"

    71, "Decrypt Keys Not Set Up"
    72, "Late Key Load Error"
    73, "Got Certificate Status msg"
    74, "RSA Key Missing Error"
    75, "Secure Renegotiation Not Supported"

    76, "Get Session Stats Failure"
    77, "Reassembly Buffer Size Exceeded"
    78, "Dropping Lost Fragment"
    79, "Dropping Partial Record"
    80, "Clear ACK Fault"

    81, "Bad Decrypt Size"
    82, "Extended Master Secret Hash Error"
    83, "Handshake Message Split Across TLS Records"
    84, "ECC Private Decode Error"
    85, "ECC Public Decode Error"

    86, "Watch callback not set"
    87, "Watch hash failed"
    88, "Watch callback failed"
    89, "Bad Certificate Message"
    90, "Store data callback not set"

    91, "No data destination Error"
    92, "Store Data callback failed"
    93, "Loading chain input"
    94, "Got encrypted extension"
    95, "Got Hello Retry Request"

    96, "Setting up keys"
    97, "Unsupported TLS Version"
    98, "Server Client Key Mismatch"
}
